#include "clockface.h"
#include "clock.h"


// ********************************************
// VARIABLE DECLARATIONS
// ********************************************

TimeFormatType timeFormat;							// Used for collecting the real time from the system
RectangleType winrectangle = { {0,16}, {160, 144} };// Used for clearing an area of screen

int latesthour=0;	//  These Variables are for detecing when a time has changed, so that the
int lastminute=-1;	// Clock does not get drawn repeatedly.

int menu_is_activated=0; // true when menu is on - used to stop the clock from drawing over the menu



// ********************************************
// 					PilotMain
// Modified 2/10/99 H.Tomlinson.
// ********************************************
DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
	int error;

	if (cmd == sysAppLaunchCmdNormalLaunch)
	{
		error = StartApplication();	// Application start code
		if (error) return error;

		EventLoop();	// Event loop

		StopApplication();	// Application stop code
	}
	return 0;
}


// ********************************************
// 				StartApplication
// Modified 2/10/99 H.Tomlinson.
// ********************************************
static int StartApplication(void)
{
	//Collect any necessary graphics (i.e. for the mini icons at the top)
	SetupGraphics();

	// Boot up the form!
	FrmGotoForm(ClockForm);

	return 0;
}



// ********************************************
// 					EventLoop
// Modified 2/10/99 H.Tomlinson.
// ********************************************
static void EventLoop(void)
{
	short err;
	int formID;
	FormPtr form;
	EventType event;


	do
	{

		EvtGetEvent(&event, 25);  // Wait for an event,
		//or if one doesn't happen by 25 ticks time, generate one anyway.
		// ( There are 100 ticks per second on a normal palm device, but don't count on it,
		// Some people over or Under clock their palms, so it can vary considerably!)


		if (SysHandleEvent(&event)) continue;
		//If its a system event, pass it to the system and drop to the end of the loop.

		if (MenuHandleEvent((void *)0, &event, &err)) continue;
		//If its a Menu event, pass it to the menu and drop to the end of the loop.

		if (event.eType == frmLoadEvent)  // If the event was that the form was just loaded,
		{
			formID = event.data.frmLoad.formID;	// Get the Id of the form loaded,
			form = FrmInitForm(formID);			// initialise the form
			FrmSetActiveForm(form);				// make the form active.
			switch (formID)
			{
				case ClockForm:					// If it was the Main Clock Form (i.e. our own)
				 FrmSetEventHandler(form, (FormEventHandlerPtr) FormHandler);
				 // Set further events to be handled by our own function, FormHandler
				break;
			}
		}
		FrmDispatchEvent(&event); // Deal with the event, whatever it was.

		draw_time();	// If the event wasn't dealt with by anything else, then it
		// must have got through its 25 ticks, in which case now would be a good time
		// to draw the clock onto the screen.

	} while(event.eType != appStopEvent);
	// If someone presses a hardware button, that will run another app, but will
	// signal this app (clock) to Stop ( with AppStopEvent ).


}



// ********************************************
// 				StopApplication
// Modified 2/10/99 H.Tomlinson.
// ********************************************
static void StopApplication(void)
{
	// Put any code here for handling things that need to be done before exiting,
	// like saving records into a database, or saving preferences etc.

	DestroyGraphics();
	// Remove the graphics from memory, since they are no longer needed
}




// **********************************************
// *                                            *
// *    Form Handler                            *
// *   Last modified 10/3/99                    *
// **********************************************

static Boolean FormHandler(EventPtr event)
{
  FormPtr   form = FrmGetActiveForm();
  int       handled = 0;
  RectangleType r = { {10,20}, {140, 120} };
  DateTimeType dateTime;

  switch (event->eType)   // Find out what sort of event it was.
  {
  case winEnterEvent:	  // If we've entered the form, its safe to draw to the screen.
	if (event->data.winEnter.enterWindow == (WinHandle)form)
	    menu_is_activated=0;
    break;

  case winExitEvent:		// If we've entered the menu, set the flag so that we don't try to draw over the menu!
	if (event->data.winExit.exitWindow == (WinHandle)form)
	    menu_is_activated=1;
    break;

  case frmOpenEvent:

    TimSecondsToDateTime (TimGetSeconds(), &dateTime);  // Get the time from the system, and convert it into a date and time.
	FrmDrawForm(form);
    handled = 1;
    break;

  case penDownEvent:  // If someone presses down somewhere on the screen, pass the x and y coordinates to the function OnPen(x,y)

	handled = OnPen(event->penDown,
				event->screenX,event->screenY);

	break;

  case ctlSelectEvent:  // A control button was pressed and released.

  case menuEvent:  // If it was a menu event....
    menu_is_activated=1;
    switch (event->data.menu.itemID)	// Find which menu item it was, and display the right form.
    {
	case OptionsAbout:
	 	ShowMessageDialog(AboutForm);
		//ShowAbout() ;
		break;
	case OptionsHelp:
		FrmAlert(HelpAlert);
		break;
    }
    handled = 1;
	menu_is_activated=0;
    break;

  case nilEvent:
    //ChangeBitmap();
    handled = 1;
    break;
  }
  return handled;
}



// **********************************************************
// *  DRAW TIME PROCEDURE - Modified 2/10/99                *
// *  Gets the system time                                  *
// *  Display time/date + icons in the top row              *
// *  calls draw_clockface to draw the actual clock         *
// **********************************************************

static void draw_time()
{

	DateTimeType dateTime;	// To store the date and time

	char tmp[50];	// Need a temporary String variable to write strings.
	char ddate[20];	// A variable to hold the string for the date.
	char dtime[20]; // A variable to hold the string for the time.

	int nowhour;	// Variables for getting info from the time/date.
	int nowminute;
	int nowsec;
    int nowday;
    int dayofmon;
	int mon;

	if (menu_is_activated)
		return;
		// So that we don't draw over the menu if it is open.

	TimSecondsToDateTime (TimGetSeconds (), &dateTime);
	nowday = dateTime.weekDay;
	nowhour=dateTime.hour;
	nowminute=dateTime.minute;
    nowsec=dateTime.second;
	dayofmon=dateTime.day;
	mon=dateTime.month;

	StrCopy(ddate,StrIToA(tmp,dayofmon));  // Make the ddate[] hold todays sate - i.e. "22/10""
	StrCat(ddate,"/");
	StrCat(ddate,StrIToA(tmp,mon));
	StrCat(ddate," ");

	StrCopy(dtime," ");		// Make dtime[] hold the time
	StrCat(dtime,StrIToA(tmp,nowhour));
	StrCat(dtime,":");

	if (nowminute<10) StrCat(dtime,"0");	// Add a '0' if the number of minutes is less than 10.
	StrCat(dtime,StrIToA(tmp,nowminute));
    StrCat(dtime,":");
	if (nowsec<10) StrCat(dtime,"0");
	StrCat(dtime,StrIToA(tmp,nowsec));



	FntSetFont(stdFont);			// Use the standard font
	drawChars(dtime,159,1,Right,1);	// Draw the time, right justified to 159,1
    drawChars(ddate,72,1,Left,1);	// Draw the date, left justified to 72,1
	// NOTE : drawChars is a function written by David Oakley to make formatting
	// of text a little easier. It is included in graphics.c


	// Copy in the nice little graphics we use for a date and time icon.
	SpriteCopy(9,1,112,2,9,10, scrCopy);  // Draws in the clock
    SpriteCopy(0,1,61,2,9,10, scrCopy);  // Draws in the date icon


	draw_clockface(); // Draw in the clock face itself.
	// this is defined in clockface.c


}


static int OnPen(int pendown,int x, int y)
{
	// Being able to handle the pen events is very useful, but clock
	//does not need to. I left it in because most apps tend to use it.
	return 0;
}




// Show a message, if necessary.
void ShowMessageDialog(Word fid)
{
	FormPtr form;
	form = FrmInitForm(fid);
	if (form)
	{
		FrmDoDialog(form);
		FrmDeleteForm(form);
	}
}

