


###############
# DEFINITIONS #
###############

# Define where to find the compiler programs
GCC = m68k-palmos-coff-gcc
PILRC = pilrc.exe

# List the object files that are needed; make will find the relevant
# .c or .cpp files to create these
OBJ =	clock.o graphics.o clockface.o

# Switch on some warnings by including $(WARNINGS) in C_FLAGS
WARNINGS = -Wunused -Wuninitialized -Wcomment -Wreturn-type 

# Set up the flags that will be passed to the compiler/linker
# -O2 for optimise
# ERROR_CHECK_FULL for lots of warnings about style, etc.
C_FLAGS = -O2 -DERROR_CHECK_LEVEL=ERROR_CHECK_NONE

# Faster code can be created at the expense of size by uncommenting:
# C_FLAGS=$(C_FLAGS) -finline-functions

# The base name of the executable is useful to create various
# intermediate files
BASENAME = Clock

# The final output file is...
OUTPUT = $(BASENAME).prc

#########
# RULES #
#########

# .o (Object) files can be created from .cpp (C++) files
.cpp.o:
	$(GCC) $(C_FLAGS) -c $*.cpp

# .o (Object) files can also be created from .c (C) files
.c.o:
	$(GCC) $(C_FLAGS) -c $*.c

# The final output (a .prc file) is created from the set of .grc
# and .bin files
$(OUTPUT): code0001.$(BASENAME).grc tver0001.bin
	build-prc.exe  $(OUTPUT) "Clock" AsCl *.grc *.bin
	@ls -l *.prc
#########################################################################################
# NOTE	AsCl (above) is the Creator ID, and must be unique for each application.
# You must register a Creator ID with palm, via www.palm.org
#########################################################################################

# Various .bin files are created from the .rcp (Resource) file and
# related .bmp (Bitmap) files
tver0001.bin: clock.rcp clock.h resource.h clock.bmp graphics.bmp
	$(PILRC) -q clock.rcp

# .grc files are created from the .o (Object) files
code0001.$(BASENAME).grc: $(OBJ)
	m68k-palmos-coff-gcc $(C_FLAGS) $** -o $(BASENAME)
	m68k-palmos-coff-obj-res $(BASENAME)



# Run NMAKE clean to remove any intermediate files
clean:
	del *.grc
	del *.bin
	del $(OBJ)
	del $(BASENAME)
