#include "Clockface.h"
#include "graphics.h"

TimeFormatType timeFormat;


// Without using 'real' maths functions which is hard work, I've defined my
// own set of cosines and sines, so that I can draw the clock face.
struct cossintype {
       int cos;
	   int sin;
};
typedef struct cossintype CossinType;


// These values are calculated for each minute hand position (1 to 60)
// The two numbers are:
//  160*cos( (minute/60) * 2 * PI )
//  160*sin( (minute/60) * 2 * PI )
// I chose it to be multiplied by 160, since that is the largest possible
// line I could want to draw on the palm screen. You'll see below that it is
// actually made shorter as wanted.

CossinType cossin[65] = {
{160,00},{159,17},{157,33},{152,49},{146,65},{139,80},
{129,94},{119,107},{107,119},{94,129},{80,139},{65,146},
{49,152},{33,157},{17,159},{00,160},{-17,159},{-33,157},
{-49,152},{-65,146},{-80,139},{-94,129},{-107,119},{-119,107},
{-129,94},{-139,80},{-146,65},{-152,49},{-157,33},{-159,17},
{-160,00},{-159,-17},{-157,-33},{-152,-49},{-146,-65},{-139,-80},
{-129,-94},{-119,-107},{-107,-119},{-94,-129},{-80,-139},{-65,-146},
{-49,-152},{-33,-157},{-17,-159},{00,-160},{17,-159},{33,-157},
{49,-152},{65,-146},{80,-139},{94,-129},{107,-119},{119,-107},
{129,-94},{139,-80},{146,-65},{152,-49},{157,-33},{159,-17},
{160,00}
};


// **********************************************************
// *  DRAW Clockface PROCEDURE - Modified 2/10/99           *
// *  Gets the system time                                  *
// *  Display a clock face on the screen                    *
// **********************************************************

void draw_clockface()
{

	DateTimeType dateTime;
	int nowhour;
	int nowminute;
	int nowsec;
    int nowday;
	int xpos;
	int ytop;
	int ybot;
	int dayofmon;
	int mon;
	int n;
    long nowtime;
	RectangleType scrrectangle = { {0,16}, {160, 144} };
    RectangleType r;
	int hourtime;
	char tmp[10];


	// Get the system time.
	TimSecondsToDateTime (TimGetSeconds (), &dateTime);
	nowday = dateTime.weekDay;
	nowhour=dateTime.hour;
	nowminute=dateTime.minute;
    nowsec=dateTime.second;
	dayofmon=dateTime.day;
	mon=dateTime.month;

	// Use the graphics.c DrawToBuffer so that we draw all the graphics to
	// a spare screen, called the buffer. When we've finished drawing,
	// we copy the buffer to the screen.
	// This prevents the flickering if we were just to draw straight to the screen.
	DrawToBuffer();

	// Erase a section on the buffer.
    WinEraseRectangle(&scrrectangle,0);



	// Draw in the numbers on the clock face, 1 to 12.
    for (n=1;n<13;n++) {
      StrCopy(tmp,StrIToA(tmp,n));
  	  drawChars(tmp,80+cossin[n*5].sin/2.4,85-cossin[n*5].cos/2.4,Centre,1);
	}

	// Call draw_hand (below) to draw in the two hands.
    draw_hand(80,90,nowminute,60,2);	// minute hand
	draw_hand(80,90,nowsec,60,1);		// second hand

	if(nowhour>12) nowhour-12;
    hourtime=(nowhour*60)+nowminute;
    hourtime=60*hourtime/(60*12)	;	// convert to 12 hour time,

    draw_hand(80,90,(int) (hourtime),40,3);	// draw in the hour hand.

	DrawToScreen();	// tell the palm to draw to the screen,
    ShowScreenArea(); // copy the buffer picture to the screen.

}


//************************************************
//	draw_hand function
// modified 2/10/99 - H.Tomlinson
//	Draws in a hand from xstart,ystart, to a particular time position (minutes equivalent)
// with a length and width, so hour, minute and second can look different.
//************************************************
static void draw_hand(short xstart,short ystart,int timepos, int length, int width) {

 long xwidth;
 long ywidth;
 short xend;
 short yend;
 int n;
 int m;

 if (timepos>59) timepos=timepos-60;


 // convert the length to a width and height, by resolving with trigonometry
 xwidth=(length*cossin[timepos].sin)/160;
 ywidth=(length*cossin[timepos].cos)/160;

 xend=xstart+xwidth; // find the end coordinates.
 yend=ystart-ywidth;


 // second hand.
 if (width==1) WinDrawLine(xstart,ystart,xend,yend); // draw a line from start to end.


 // minute hand
 if (width==2) {
      // draw several lines - makes a nice triangle.
      WinDrawLine(xstart,ystart,xend,yend);
      WinDrawLine(xstart-1,ystart,xend,yend);
      WinDrawLine(xstart+1,ystart,xend,yend);
      WinDrawLine(xstart,ystart-1,xend,yend);
      WinDrawLine(xstart,ystart+1,xend,yend);

 }


 // hour hand
 if (width==3) {
      // lots of lines. This makes sure that the hour hand looks solid.
      WinDrawLine(xstart,ystart,xend,yend);
      WinDrawLine(xstart-1,ystart,xend,yend);
      WinDrawLine(xstart+1,ystart,xend,yend);
      WinDrawLine(xstart,ystart-1,xend,yend);
      WinDrawLine(xstart,ystart+1,xend,yend);
      WinDrawLine(xstart-1,ystart-1,xend,yend);
      WinDrawLine(xstart+1,ystart+1,xend,yend);
      WinDrawLine(xstart+1,ystart-1,xend,yend);
      WinDrawLine(xstart-1,ystart+1,xend,yend);

   }



}


