#include "graphics.h"


// Create handles for three graphics, the screen, a buffer, and the graphics we have made ourself,
WinHandle	realwh = 0,
			bufferwh=0,
			graphicswh = 0;


// draw a bitmap at x,y
// This function is not used directly, but we have made a 'wrapper' function to make it neater.
void DrawBitmap(Int x, Int y, Int id)
{
    VoidHand h;
    BitmapPtr p;
    h = DmGet1Resource('Tbmp', id);
    if (h != NULL) {
        p = (BitmapPtr) MemHandleLock(h);
        WinDrawBitmap(p, x, y);
        MemHandleUnlock(h);
        DmReleaseResource(h);
    }
}


// This function copies the buffer graphic to the screen.
void ShowScreenArea(void)
	{

		static RectangleType r = { 0,16,160,160 };

		WinCopyRectangle(bufferwh,realwh,
						 &r,0,16,
						 scrCopy);
}


// sets the current drawing window to be the buffer.
void DrawToBuffer(void)
	{
	  WinSetDrawWindow(bufferwh);
}

// sets the current drawing window to be the screen. (the REAL window!)
void DrawToScreen(void)
	{
	  WinSetDrawWindow(realwh);
}

void SetupGraphics()
{
	// Called by StartApplication in clock.c
	Word error;


	int graphicsW = 19,graphicsH = 11;
	// The size of the actual graphic (bitmap) we want to include.

	realwh = WinGetDrawWindow();
	// Find the handle, so we can refer to the real window.

	graphicswh = WinCreateOffscreenWindow(graphicsW,graphicsH,screenFormat,&error);    // Need exact size!
	// Create an area for the graphic to live in.
	ErrFatalDisplayIf(error, "Error creating graphics buffer");
	// Create an area for the graphic to live in.

	WinSetDrawWindow(graphicswh);
	// Set the current drawing window to be the graphic,

	DrawBitmap(0,0,GraphicsBitmap);
	// copy the bitmap 'GraphicsBitmap' (which is defined in resource.h) to the current window (the graphic window)

	bufferwh = WinCreateOffscreenWindow(160,160,screenFormat,&error);
	// Create a buffer area for drawing later.
		ErrFatalDisplayIf(error, "Error creating off-screen buffer");

	WinSetDrawWindow(realwh);
	// Set the drawing window back to being the real screen.

}

void DestroyGraphics()
{
	WinDeleteWindow(graphicswh,false);
	WinDeleteWindow(bufferwh,false);
	// Delete the two windows we created at the start, and release their resources back to the system.

}




// **********************************************************
// *  SpriteCopy Function  - Modified 2/10/99               *
// *  Copies a sprite (a small graphic)                     *
// *  from the graphic window to the current drawing area.  *
// **********************************************************
void SpriteCopy(int srcX,int srcY,int destX,int destY,int width,int height,int op)
{
	RectangleType r;

	r.topLeft.x = srcX;
	r.topLeft.y = srcY;
	r.extent.x = width;
	r.extent.y = height;
	WinCopyRectangle(graphicswh,realwh,&r,destX,destY,op);
}


// **************************************************************************
// *  DrawChars - David Oakley	- 1998					    			    *
// *  Allows copying of characters to the screen with particular alignment  *
// **************************************************************************
void drawChars(char *str,int x,int y,Alignment align,Boolean black)
{

	int len = StrLen(str);

	// align could be : Left,Centre,Right
	if (align != Left)
	{
		int width = FntCharsWidth(str,len);
		x -= (align == Centre) ? width/2 : width;
	}

	if (black)
		WinDrawChars(str,len,x,y);
	else
		WinEraseChars(str,len,x,y);
}
