//
// SMatches is a freeware program for HSPascal.
// SMatches Version 1.1 - October 30, 2001 - Made by Laurent Duveau
//       Web Site = http://aldweb.free.fr
//       e-Mail   = aldweb@free.fr
//
// SMatches is based on the source code of Matches freeware.
// Matches Version 1.4 - August 6, 2001 - Made by Laurent Duveau
//
// SMatches was compiled with the great HSPascal compiler
// for Palm OS handheld devices
//       Web Site = http://HSPascal.Fihl.net
//       e-Mail   = Christen@Fihl.net
//
// The original source code of Matches was slightly modified  
// for Speedy tests purposes, and the program was renamed 
// as SMatches for no confusion making.
//


Program SMatches;

{$SearchPath Units; Units\UI; Units\System}
{$ApplName SMatches,LDSM}
// LDSM is NOT a referenced PalmOS CreatorID


Uses
  Window, Form, Menu, Rect, Event, SysEvent, SystemMgr, FloatMgr, HSUtils, 
  TimeMgr, Preferences, SystemResources;

Resource
  MenuRes=(ResMBAR,,
    (ResMPUL,(6,14,80,22),(4,0,35,12),'Game',   
      MenuNewGame=(,'N','New game'),
      MenuQuit=(,'X','Exit')),
    (ResMPUL,(42,14,70,33),(40,0,30,12),'Help',  
      MenuRules=(,'R','Rule'),
      MenuTips=(,'T','Tip'),
      MenuAbout=(,'A','About')));

  MainForm=(ResTFRM,1000,(0,0,160,160),0,0,0, (FormTitle,'SMatches'),
    Main1Button=(FormButton,,(16,145,36,12),,,,'1'),
    Main2Button=(FormButton,,(60,145,36,12),,,,'2'),
    Main3Button=(FormButton,,(104,145,36,12),,,,'3'));

  Rule1=(ResTalt,,0,0,0,'Matches - Rule','Matches is a silly little game in '+
         'which each of the two players (computer and you) removes one after the other from one to '+
         'three matches.'+#10+'The player who removes the last match looses the game.','Ok');
  Tip1=(ResTalt,,0,0,0,'Matches - Tip','There is a way to win every time. Try to '+
        'figure out what rule you should follow to make sure you win against the '+
        'computer player...','Ok');
  HelpAbout1=(ResTSTR,,'Matches is a sample freeware program compiled with the great '+
              'HSPascal compiler for Palm OS handheld devices.'#10#10+ 
              'HSPascal can be found there:'#10'http://HSPascal.Fihl.net');
  About1=(ResTalt,,0,HelpAbout1,0,'Matches - About', 'Matches  v1.4'#10+
          #9'August 6, 2001'#10#9'by Laurent Duveau'#10#10+
          'Visit my Web site:'#10'http://aldweb.free.fr'#10#10+
          'e-Mail me:'#10'aldweb@free.fr','Ok');
  StartNewGame1=(ResTalt,,0,0,0,'Matches - New game','Do you really want to start '+
                 'a new game ?','Yes','No');
  First1=(ResTalt,,0,0,0,'Matches - First Player','Please choose who will start '+
          'first to remove matches:','Computer','You');
  ComputerWin1=(ResTalt,,0,0,0,'Matches - Winner','Sorry, I win !','Ok');
  PlayerWin1=(ResTalt,,0,0,0,'Matches - Winner','Good job, YOU win.','Ok');
  StartNewGame2=(ResTalt,,0,0,0,'Matches - New Game','Do you wish to play again ?','Yes','No');  

  ResultWindow=(ResTalt,,0,0,0,'SMatches - Timer Results','^1 ^2 ^3','Ok');

  (ResTAIN,1000,'SMatches');
  (ResTVER,1000,'1.1');

Const
  CreatorID=$4C44534D;      // LDSM
  Nobody=0;
  Computer=1;
  Player=2;

Var
  NbOfMatches: LongInt;
  FirstPlayer: Integer;
  MyMenu: MenuBarPtr;
  DoStop: Boolean;
  WorkRect: RectangleType;
  FormLoad,FormOpen: Boolean;
  FirstLaunch: Boolean;
  PlayerIcon : Array[1..58] of String[28];
  MatchIcon : Array[1..23] of String[7]; 
  DlySeed: LongInt;
  RndSeed : Real;
  TurnsToPlay: Integer;

Procedure StartTimer;
begin
  DlySeed:=TimGetSeconds;
end;

Function StopTimer: LongInt;
begin
  StopTimer:=TimGetSeconds-DlySeed;
end;

Function Rnd(Rdm: Integer): Integer;
begin
  RndSeed:=221*RndSeed+0.2113;
  RndSeed:=RndSeed-Trunc(RndSeed);
  Rnd:=Trunc(Rdm*RndSeed);
end;


Procedure EraseLogo(X,Y,W,H: Integer);
Var
  Rec: RectangleType;
begin
  With Rec do begin
    topLeft.X := X;
    topLeft.y := Y;
    extent.x  := W;
    extent.y  := H;
    WinEraseRectangle(Rec,0);          
  end;
end;

Procedure DrawPlayerIcon(WhoPlays,X,Y: Integer);
Var
  I,J: Integer;
  S : String;
begin
  Eraselogo(X,Y,29,29);
  for J:=0 to 28 do begin
    S:=PlayerIcon[29*(WhoPlays-1)+J+1]; 
    for I:=0 to 28 do 
      if S[I+1]='1' then 
        WinDrawLine(X+I,Y+J,X+I,Y+J);
  end;
end; 

Procedure DrawMatchIcon(ShowIt: Boolean; X,Y: Integer);
Var
  I,J: Integer;
  S: String;
begin
  if ShowIt=True then
    for J:=0 to 22 do begin
      S:=MatchIcon[J+1]; 
      for I:=0 to 6 do 
        if S[I+1]='1' then 
          WinDrawLine(X+I,Y+J,X+I,Y+J)
    end
    else Eraselogo(X,Y,7,23);
end;

Procedure DrawMatches;
Var
  N,X,Y: Integer;
begin
  for N := 0 to 29 do begin
    Y:=N div 10;
    X:=N-10*Y;
    Y:=26*Y+20;
    X:=12*X+20;
    if N<NbOfMatches then 
      DrawMatchIcon(True,X,Y)
    else
      DrawMatchIcon(False,X,Y);      
  end;
end;

Procedure DrawGameBoard(NbOfRemoved,WhoRemoved: Integer);
Var
  S,W: String;
  N: Integer;
begin
  if WhoRemoved>Nobody then begin
    Str(NbOfRemoved:1,S);
    case WhoRemoved of
      Computer: W:='I';
      Player: W:='You';
    end;
    S:=W+' remove '+S;
    if NbOfRemoved=1 then
      S:=S+' match'
    else
      S:=S+' matches';
    WinDrawChars(S,Length(S),40,125);
  end;
  NbOfMatches:=NbOfMatches-NbOfRemoved;
  DrawMatches;
  N:=SysTaskDelay(SysTicksPerSecond div 2);
  Str(NbOfMatches:2,S);
  if NbOfMatches=1 then
    S:='There is 1 match left         ' 
  else
    S:='There are '+S+' matches left '; 
  WinDrawChars(S,Length(S),35,115);
  if WhoRemoved>Nobody then begin
    S:='                                               ';
    WinDrawChars(S,Length(S),40,125);
  end;
end;

Procedure ComputerPlayerPlays;
Var
  N: Integer;
  S: String;
  M: Integer;
begin
  DrawPlayerIcon(Computer,2,110);
  if (NbOfMatches-1)/4=int((NbOfMatches-1)/4) then 
    repeat
      N:=Rnd(3)+1;
    until N<NbOfMatches
  else begin                       
    // "magic" formula is  NbOfMatches-(4*Trunc(NbOfMatches/4-0.5)+1);
    N:=Trunc(NbOfMatches/4-0.5);
    N:=4*N+1;
    N:=NbOfMatches-N;
  end;  
  DrawGameBoard(N,Computer);
  if NbOfMatches=1 then begin
//    N:=FrmAlert(ComputerWin1);
    NbOfMatches:=0;
  end;
  DrawPlayerIcon(Player,2,110);
end;

Procedure PlayerPlays(NbOfRemoved: Integer);
Var
  N: Integer;
begin
  if NbOfRemoved<NbOfMatches then begin
    DrawGameBoard(NbOfRemoved,Player);
    if NbOfMatches=1 then begin
//      N:=FrmAlert(PlayerWin1);
      NbOfMatches:=0;
    end 
    else 
      ComputerPlayerPlays;
  end;
end;

Procedure InitGame;
Var
  Init: Boolean;
begin
  Init:=True;
//  if NbOfMatches>0 then 
//    Init:=FrmAlert(StartNewGame1)<>1;
  if Init then begin
    NbOfMatches:=Rnd(29)+2;
    WinEraseRectangle(WorkRect, 0);
    DrawGameBoard(0,Nobody);
//    if FrmAlert(First1)=0 then 
//      ComputerPlayerPlays
//    else 
//      DrawPlayerIcon(Player,2,110);;
    ComputerPlayerPlays;
  end;
end;

Procedure SavePreferences;
begin
  PrefSetAppPreferences(CreatorID,0,0,@NbOfMatches,SizeOf(NbOfMatches),True);
end;

Procedure LoadPreferences;
Var
  Size,N: Integer;
begin
  Size:=SizeOf(NbOfMatches);
  N:=PrefGetAppPreferences(CreatorID,0,@NbOfMatches,Size,True);
  if N=noPreferenceFound then
    NbOfMatches:=0;
end;

Procedure InitVar;
begin
  DoStop:=False;
//  LoadPreferences;
  NbOfMatches:=0;
  FormLoad:=False;
  FormOpen:=False;
  FirstLaunch:=True;
  PlayerIcon[1]:='0000011111111111111110000000';      // Palm player
  PlayerIcon[2]:='0000111111111111111111000000';
  PlayerIcon[3]:='0000110000000000000000100000';
  PlayerIcon[4]:='0000110011111111111100100000';
  PlayerIcon[5]:='0000110111111111111110100000';
  PlayerIcon[6]:='0000110111111111111110100000';
  PlayerIcon[7]:='0000110111111111111110100000';
  PlayerIcon[8]:='0000110111111111111110100000';
  PlayerIcon[9]:='0000110111111111111110100000';
  PlayerIcon[10]:='0000110111111111111110100000';
  PlayerIcon[11]:='0000110111111111111110100000';
  PlayerIcon[12]:='0000110111111111111110100000';
  PlayerIcon[13]:='0000110111111111111110100000';
  PlayerIcon[14]:='0000110111111111111110100000';
  PlayerIcon[15]:='0000110111111111111110100000';
  PlayerIcon[16]:='0000110111111111111110100000';
  PlayerIcon[17]:='0000110111111111111110100000';
  PlayerIcon[18]:='0000110101010101010110100000';
  PlayerIcon[19]:='0000110110101010101010100000';
  PlayerIcon[20]:='0000110101010101010110100000';
  PlayerIcon[21]:='0000110110101010101010100000';
  PlayerIcon[22]:='0000110011111111111110100000';
  PlayerIcon[23]:='0000110000000000000000100000';
  PlayerIcon[24]:='0000110000000110000000100000';
  PlayerIcon[25]:='0000110110110110110110100000';
  PlayerIcon[26]:='0000110110110110110110100000';
  PlayerIcon[27]:='0000110000000110000000100000';
  PlayerIcon[28]:='0000010000000000000000100000';
  PlayerIcon[29]:='0000001111111111111111100000';
  PlayerIcon[30]:='0000000000011111111000000000';     // Human player
  PlayerIcon[31]:='0000000011111111111110000000';  
  PlayerIcon[32]:='0000000111111111111111100000';
  PlayerIcon[33]:='00000011111111111111111100000';
  PlayerIcon[34]:='00000111111111111111111110000';
  PlayerIcon[35]:='00000111111111111111111110000';
  PlayerIcon[36]:='00001111111111111111111111000';
  PlayerIcon[37]:='00001111111111111110111111000';
  PlayerIcon[38]:='00011111111111111100111111100';
  PlayerIcon[39]:='00011111111111111000011111100';
  PlayerIcon[40]:='00111111111111100000001111110';
  PlayerIcon[41]:='00111111111110000000000011110';
  PlayerIcon[42]:='01111111110000000000000001110';
  PlayerIcon[43]:='11111100000000000000000000110';
  PlayerIcon[44]:='11110001100000000000011000010';
  PlayerIcon[45]:='11000010010000000000100100011';
  PlayerIcon[46]:='10000011110000110000111100001';
  PlayerIcon[47]:='10000011110001111000111100001';
  PlayerIcon[48]:='10000001100011111100011000001';
  PlayerIcon[49]:='10000000000011111100000000001';
  PlayerIcon[50]:='11000000000001111000000000011';
  PlayerIcon[51]:='01100000110000110000110000110';
  PlayerIcon[52]:='00110000111000000001110001100';
  PlayerIcon[53]:='00011000011100000011100011000';
  PlayerIcon[54]:='00001100001111111111000110000';
  PlayerIcon[55]:='00000110000111111110001100000';
  PlayerIcon[56]:='00000011000000000000011000000';
  PlayerIcon[57]:='00000001100000000000110000000';
  PlayerIcon[58]:='00000000111111111111100000000'; 
  MatchIcon[1]:='0011100';
  MatchIcon[2]:='0100010';
  MatchIcon[3]:='1000001';
  MatchIcon[4]:='1000001';
  MatchIcon[5]:='0100010';
  MatchIcon[6]:='0111110';
  MatchIcon[7]:='0101010';
  MatchIcon[8]:='0101010';
  MatchIcon[9]:='0101010';
  MatchIcon[10]:='0101010';
  MatchIcon[11]:='0101010';
  MatchIcon[12]:='0101010';
  MatchIcon[13]:='0101010';
  MatchIcon[14]:='0101010';
  MatchIcon[15]:='0101010';
  MatchIcon[16]:='0101010';
  MatchIcon[17]:='0101010';
  MatchIcon[18]:='0101010';
  MatchIcon[19]:='0101010';
  MatchIcon[20]:='0101010';
  MatchIcon[21]:='0101010';
  MatchIcon[22]:='0011100';
  MatchIcon[23]:='0001000';
end;

Function HandleEvent(var Event: EventType): Boolean;
Var
  N: Integer;
  PForm: FormPtr;
begin
  HandleEvent:=True;
  with Event do 
  case eType of
  frmLoadEvent:
    begin
      PForm:=FrmInitForm(data.frmLoad.FormID);
      FrmSetActiveForm(PForm); 
      FrmSetEventHandlerNONE(PForm);  // Really needed? Which max PalmOS version does need it?
      FormLoad:=True;
    end;
  frmOpenEvent: 
    begin
      FrmDrawForm(FrmGetActiveForm);
      DrawGameBoard(0,Nobody);
      if NbOfMatches>0 then 
        DrawPlayerIcon(Player,2,110);
      FormOpen:=True;
    end;
  ctlSelectEvent: 
    case Data.CtlEnter.ControlID of
      Main1Button: PlayerPlays(1);
      Main2Button: PlayerPlays(2);
      Main3Button: PlayerPlays(3);
    else 
      HandleEvent:=False;
    end;
  menuEvent:
    case Data.Menu.ItemID of
      MenuNewGame: InitGame;
      MenuQuit:    DoStop:=True;
      MenuRules:   N:=FrmAlert(Rule1);
      MenuTips:    N:=FrmAlert(Tip1);
      MenuAbout:   N:=FrmAlert(About1);
    end;
  else
    HandleEvent:=False;
  end;
end;

Procedure Main;
Var
  Event: EventType;
  Error: UInt16;
  N: Integer;
begin
  RctSetRectangle(WorkRect, 0, 16, 160, 128);
  FrmGotoForm(MainForm);
  Repeat
    EvtGetEvent(Event, 5);
    if not SysHandleEvent(Event) then
      if not MenuHandleEvent(MyMenu,Event,Error) then
        if not FrmDispatchEvent(Event) then
          if not HandleEvent(Event) then 
            if (FormLoad and FormOpen) then
              if (NbOfMatches=0) then 
                if not FirstLaunch then 
                  if TurnsToPlay>0 then begin
                    InitGame;
                    dec(TurnsToPlay);
                  end
                  else 
                    DoStop:=True
                else begin
                  InitGame;
                  FirstLaunch:=False;
                end
              else begin
                repeat
                  N:=Rnd(3)+1;
                until N<NbOfMatches;
                PlayerPlays(N);
              end;
  Until DoStop; // or (Event.eType=appStopEvent);
//  SavePreferences;
  if FrmGetActiveForm<>NIL then begin
    FrmEraseForm(FrmGetActiveForm);
    FrmDeleteForm(FrmGetActiveForm);
  end;
end;

Procedure SpeedBeg;
begin
  RndSeed:=0.1234;
  TurnsToPlay:=9;
end;

Procedure SpeedEnd;
Var
  N: Integer;
begin
  N:=StopTimer;
  N:=frmCustomAlert(ResultWindow,'SMatches test lasted ',i2s(N),' seconds.');
end;

begin
  StartTimer;
  Randomize;
  InitVar;
  SpeedBeg;
  Main;
  SpeedEnd;
end.
