//
// SNumerus is a freeware program for HSPascal.
// SNumerus Version 1.1 - October 30, 2001 - Made by Laurent Duveau
//       Web Site = http://aldweb.free.fr
//       e-Mail   = aldweb@free.fr
//
// SNumerus is based on the source code of Numerus freeware.
// Numerus Version 1.2 - August 6, 2001 - Made by Laurent Duveau
//
// SNumerus was compiled with the great HSPascal compiler
// for Palm OS handheld devices
//       Web Site = http://HSPascal.Fihl.net
//       e-Mail   = Christen@Fihl.net
//
// The original source code of Numerus was slightly modified  
// for Speedy tests purposes, and the program was renamed 
// as SNumerus for no confusion making.
//


Program SNumerus;    

{$SearchPath Units; Units\UI; Units\System}
{$ApplName SNumerus,LDSN}
// LDSN is NOT a referenced PalmOS CreatorID


Uses
  Window, Form, Menu, Rect, Event, SysEvent, SystemMgr, FloatMgr, HSUtils, Font,
  Preferences, SystemResources, TimeMgr;

Resource
  MenuRes=(ResMBAR,,
    (ResMPUL,(6,14,75,22),(4,0,31,12),'Appl',   
      MenuReset=(,'R','Reset'),
      MenuQuit=(,'X','Exit')),
    (ResMPUL,(42,14,75,33),(40,0,31,12),'Help',  
      MenuHelp=(,'H','Help'),
      MenuTips=(,'T','Tip'),
      MenuAbout=(,'A','About')));

  MainForm=(ResTFRM,1000,(0,0,160,160),0,0,0, (FormTitle,'SNumerus'),
    MCButton=(FormButton,,(2,54,20,17),,,,'MC'),
    MRButton=(FormButton,,(2,75,20,17),,,,'MR'),
    MPButton=(FormButton,,(2,96,20,17),,,,'M+'),
    CEButton=(FormButton,,(2,118,20,18),,,,'CE'),
    CButton=(FormButton,,(2,140,20,18),,,,'C'),
    NXButton=(FormButton,,(26,54,32,32),,,,'X'),
    NVButton=(FormButton,,(26,90,32,32),,,,'V'),
    NIButton=(FormButton,,(26,126,32,32),,,,'I'),
    NCButton=(FormButton,,(62,54,32,32),,,,'C'),
    NLButton=(FormButton,,(62,90,32,32),,,,'L'),
    EQUButton=(FormButton,,(62,126,68,32),,,,'='),
    NMButton=(FormButton,,(98,54,32,32),,,,'M'),
    NDButton=(FormButton,,(98,90,32,32),,,,'D'),
    DIVButton=(FormButton,,(134,54,24,22),,,,'/'),
    MULButton=(FormButton,,(134,80,24,22),,,,'X'),
    SUBButton=(FormButton,,(134,106,24,22),,,,'-'),
    ADDButton=(FormButton,,(134,132,24,26),,,,'+'));

  Help1=(ResTalt,,0,0,0,'Numerus - Help','Numerus is a basic standard 4 function '+
         'calculator... that operates in Roman numerals.'#10#10+
         'Numerus accepts numbers in the range {0..99999}','Ok');
  Tip1=(ResTalt,,0,0,0,'Numerus - Tip','I = 1'#9#9#9'V = 5'#10'X = 10'#9#9'L = 50'#10+
        'C = 100'#9#9'D = 500'#10'M = 1000'#10#10'For numbers > 4999 :'#10+
        'N.M means N x 1000'#10'For example :'#10'V.M is 5000','Ok');
  HelpAbout1=(ResTSTR,,'Numerus is a sample freeware program compiled with the great '+
              'HSPascal compiler for Palm OS handheld devices.'#10#10+
              'HSPascal can be found there:'+#10+'http://HSPascal.Fihl.net');
  About1=(ResTalt,,0,HelpAbout1,0,'Numerus - About', 'Numerus  v1.2'#10+
          #9'August VI, MMI'#10#9'by Laurent Duveau'#10#10+
          'Visit my Web site:'#10'http://aldweb.free.fr'#10#10+
          'e-Mail me:'#10'aldweb@free.fr','Ok');

  ResultWindow=(ResTalt,,0,0,0,'SNumerus - Timer Results','^1 ^2 ^3','Ok');
  
  (ResTAIN,1000,'SNumerus');
  (ResTVER,1000,'1.1');

Type
  TOper = (cEQU,cADD,cSUB,cMUL,cDIV);
  TMemDraw = (mONE,mTWO);
  TPref = Record
            N1: LongInt;
            N2: LongInt;
            MD: TMemDraw;
            NM: LongInt;
            OP: TOper;
          end;

Const
  CreatorID=$4C44534E;      // LDSN

Var
  MyMenu: MenuBarPtr;
  DoStop: Boolean;
  WorkRect: RectangleType;
  FormLoad,FormOpen: Boolean;
  NumArab1: LongInt;
  NumArab2: LongInt;
  MemDraw: TMemDraw;
  NumMem: LongInt;
  Oper: TOper;
  DlySeed: LongInt;
  RndSeed : Real;
  TurnsToPlay: Integer;

Procedure StartTimer;
begin
  DlySeed:=TimGetSeconds;
end;

Function StopTimer: LongInt;
begin
  StopTimer:=TimGetSeconds-DlySeed;
end;

Function Rnd(Rdm: Integer): Integer;
begin
  RndSeed:=221*RndSeed+0.2113;
  RndSeed:=RndSeed-Trunc(RndSeed);
  Rnd:=Trunc(Rdm*RndSeed);
end;

Procedure CReset;
begin
  NumArab1:=0;
  NumArab2:=0;
  MemDraw:=mONE;
  Oper:=cEQU;
end;

Procedure SavePreferences;
Var
  ToSave: TPref;
begin
  with ToSave do begin
    N1:=NumArab1;
    N2:=NumArab2;
    MD:=MemDraw;
    NM:=NumMem;
    OP:=Oper;
  end;
  PrefSetAppPreferences(CreatorID,0,0,@ToSave,SizeOf(ToSave),True);
end;

Procedure LoadPreferences;
Var
  ToLoad: TPref;
  Size,N: Integer;
begin
  Size:=SizeOf(ToLoad);
  N:=PrefGetAppPreferences(CreatorID,0,@ToLoad,Size,True);
  if N=noPreferenceFound then begin
    CReset;
    NumMem:=0;
  end
  else
    with ToLoad do begin
      NumArab1:=N1;
      NumArab2:=N2;
      MemDraw:=MD;
      NumMem:=NM;
      Oper:=OP;
    end;
end;

Function CalcNumLatin(NumArab: LongInt): String;
Const
  One = 'I';
  Five = 'V';
  Ten = 'X';
  Fifty = 'L';
  OneHundred = 'C';
  FiveHundred = 'D';
  OneThousand = 'M';
Var
  NumLatin: String;
  Procedure AddLatin(LatinChar: String);
  begin
    NumLatin:=LatinChar+NumLatin;
  end;
  Procedure AddRoman(NumAr: LongInt);
  Var
    N1,N2,N3,N4: Integer;
  begin
    N1:= NumAr mod 10;
    Case N1 of
      1: AddLatin(One);
      2: AddLatin(One+One);
      3: AddLatin(One+One+One);
      4: AddLatin(One+Five);
      5: AddLatin(Five);
      6: AddLatin(Five+One);
      7: AddLatin(Five+One+One);
      8: AddLatin(Five+One+One+One);
      9: AddLatin(One+Ten);
    end;
    N2 := NumAr mod 100 - N1;
    Case N2 of
      10: AddLatin(Ten);
      20: AddLatin(Ten+Ten);
      30: AddLatin(Ten+Ten+Ten);
      40: AddLatin(Ten+Fifty);
      50: AddLatin(Fifty);
      60: AddLatin(Fifty+Ten);
      70: AddLatin(Fifty+Ten+Ten);
      80: AddLatin(Fifty+Ten+Ten+Ten);
      90: AddLatin(Ten+OneHundred);
    end;
    N3 := NumAr mod 1000 - N2 - N1;
    Case N3 of
      100: AddLatin(OneHundred);
      200: AddLatin(OneHundred+OneHundred);
      300: AddLatin(OneHundred+OneHundred+OneHundred);
      400: AddLatin(OneHundred+FiveHundred);
      500: AddLatin(FiveHundred);
      600: AddLatin(FiveHundred+OneHundred);
      700: AddLatin(FiveHundred+OneHundred+OneHundred);
      800: AddLatin(FiveHundred+OneHundred+OneHundred+OneHundred);
      900: AddLatin(OneHundred+OneThousand);
    end;
    N4 := NumAr mod 10000 -N3 - N2 - N1;
    Case N4 of
      1000: AddLatin(OneThousand);
      2000: AddLatin(OneThousand+OneThousand);
      3000: AddLatin(OneThousand+OneThousand+OneThousand);
      4000: AddLatin(OneThousand+OneThousand+OneThousand+OneThousand);
    end;
  end;
begin
  NumLatin:='';
  if NumArab>4999 then begin
    AddRoman(NumArab mod 1000);
    AddLatin('.'+OneThousand+'  ');    
    AddRoman(NumArab div 1000);
  end
  else AddRoman(NumArab);
  CalcNumLatin:=NumLatin;
end;

Procedure DrawNumbers(MD: TMemDraw);
Var
  NumArab: LongInt;
  NumLatin: String;
  UL: UnderlineModeType;
  FID: FontID;
  T: String;
  N: Integer;
  Procedure CleanFrame;
  Var
    Rec: RectangleType;
  begin
    With Rec do begin
      topLeft.X := 6;
      topLeft.y := 22;
      extent.x  := 148;
      extent.y  := 8;
      WinEraseRectangle(Rec,0);
      topLeft.y := 35;
      extent.y  := 12;
      WinEraseRectangle(Rec,0);
    end;
  end;
begin
  MemDraw:=MD;
  case MemDraw of
    mONE: NumArab:=NumArab1;
    mTWO: NumArab:=NumArab2;
  end;
  CleanFrame;
  FID:=FntSetFont(boldFont);  // can be stdFont, boldFont or largeBoldFont
  if NumMem>0 then 
    WinDrawChars('m',1,6,19);
  if (NumArab<0) or (NumArab>99999) then begin
    T:='error';
    N:=FntCharsWidth(T,Length(T));
    WinDrawChars(T,Length(T),91-N,20);
    CReset;     
  end
  else begin
    Str(NumArab:6,T);
    N:=FntCharsWidth(T,Length(T));
    WinDrawChars(T,Length(T),91-N,20);
    FID:=FntSetFont(largeBoldFont);
    NumLatin:=CalcNumLatin(NumArab);
    WinDrawChars(NumLatin,Length(NumLatin),6,34);
  end;
end;

Procedure InitCalc;
  Procedure DrawFrame;
  Var
    Rec: RectangleType;
  begin
    With Rec do begin
    topLeft.X := 3;
    topLeft.y := 19;
    extent.x  := 154;
    extent.y  := 30;
    WinDrawRectangleFrame(dialogFrame,Rec);
    topLeft.y := 32;
    extent.y  := 2;
    WinDrawRectangle(Rec,0);
    end;
  end;
begin
  DoStop:=False;
  FormLoad:=False;
  FormOpen:=False;
  DrawFrame;
  DrawNumbers(MemDraw);
end;

Procedure AddArab2(N: LongInt);
begin
  inc(NumArab2,N);
  DrawNumbers(mTWO);
end;

Procedure Calculate(C: TOper);
begin
  case Oper of
    cADD: inc(NumArab1,NumArab2);
    cSUB: dec(NumArab1,NumArab2);
    cMUL: NumArab1:=NumArab1*NumArab2;
    cDIV: NumArab1:=NumArab1 div NumArab2;
    cEQU: if NumArab2>0 then 
            NumArab1:=NumArab2;
  end;
  DrawNumbers(mONE);
  Oper:=C;
  NumArab2:=0;
end;

Function HandleEvent(var Event: EventType): Boolean;
var
  N: Integer;
  PForm: FormPtr;
begin
  HandleEvent:=True;
  with Event do 
  case eType of
  frmLoadEvent:
    begin
      PForm:=FrmInitForm(data.frmLoad.FormID);
      FrmSetActiveForm(PForm); 
      FrmSetEventHandlerNONE(PForm);  // Really needed? Which max PalmOS version does need it?
      FormLoad:=True;
    end;
  frmOpenEvent: 
    begin
      FrmDrawForm(FrmGetActiveForm);
      FormOpen:=True;
    end;
  ctlSelectEvent: 
    case Data.CtlEnter.ControlID of
      MCButton:  begin
                   NumMem:=0;
                   if NumArab2>0 then
                     DrawNumbers(mTWO)
                   else 
                     DrawNumbers(mONE);
                 end;
      MRButton:  if Oper=cEQU then begin
                   NumArab1:=NumMem;
                   DrawNumbers(mONE);
                 end
                 else begin
                   NumArab2:=NumMem;
                   DrawNumbers(mTWO);
                 end;
      MPButton:  begin
                   if NumArab2>0 then begin
                     inc(NumMem,NumArab2);
                     NumArab1:=NumArab2;
                     NumArab2:=0;
                   end
                   else 
                     inc(NumMem,NumArab1);
                   DrawNumbers(mONE);
                 end;
      CEButton:  begin
                   NumArab2:=0;
                   DrawNumbers(mTWO);
                 end;
      CButton:   begin
                   CReset;
                   DrawNumbers(mONE);                   
                 end;
      NXButton:  AddArab2(10);
      NVButton:  AddArab2(5);
      NIButton:  AddArab2(1);
      NCButton:  AddArab2(100);
      NLButton:  AddArab2(50);
      NMButton:  AddArab2(1000);
      NDButton:  AddArab2(500);
      DIVButton: Calculate(cDIV);
      MULButton: Calculate(cMUL);
      SUBButton: Calculate(cSUB);
      ADDButton: Calculate(cADD);
      EQUButton: Calculate(cEQU);
    else 
      HandleEvent:=False;
    end;
  menuEvent:
    case Data.Menu.ItemID of
      MenuReset:  begin
                    CReset;
                    NumMem:=0;
                    InitCalc;
                  end;
      MenuQuit:   DoStop:=True;
      MenuHelp:   N:=FrmAlert(Help1);
      MenuTips:   N:=FrmAlert(Tip1);
      MenuAbout:  N:=FrmAlert(About1);
    end;
  else
    HandleEvent:=False;
  end;
end;

Procedure DoCalc;
Var
  N: Integer;
begin
  N:=Rnd(100);
  case N of
    96,97,98,99:  begin
              NumMem:=0;
              if NumArab2>0 then
              DrawNumbers(mTWO)
              else 
                DrawNumbers(mONE);
            end;
    91,92,93,94,95:  if Oper=cEQU then begin
              NumArab1:=NumMem;
              DrawNumbers(mONE);
            end
            else begin
              NumArab2:=NumMem;
              DrawNumbers(mTWO);
            end;
    86,87,88,89,90:  begin
              if NumArab2>0 then begin
                inc(NumMem,NumArab2);
                NumArab1:=NumArab2;
                NumArab2:=0;
              end
              else 
                inc(NumMem,NumArab1);
              DrawNumbers(mONE);
            end;
    81,82,83,84,85:  begin
              NumArab2:=0;
              DrawNumbers(mTWO);
            end;
    76,77,78,79,80:   begin
              CReset;
              DrawNumbers(mONE);                   
            end;
    21,22,23,24,25,26,27,28,29,30:  AddArab2(10);
    11,12,13,14,15,16,17,18,19,20:  AddArab2(5);
    1,2,3,4,5,6,7,8,9,10:  AddArab2(1);
    41,42,43,44,45:  AddArab2(100);
    31,32,33,34,35,36,37,38,39,40:  AddArab2(50);
    100:  AddArab2(1000);
    46,47,48,49,50:  AddArab2(500);
    51,52,53,54,55: Calculate(cDIV);
    56,57,58,59,60: Calculate(cMUL);
    61,62,63,64,65: Calculate(cSUB);
    66,67,68,69,70: Calculate(cADD);
    71,72,73,74,75: Calculate(cEQU);
  end;
end;

Procedure Main;
var
  Event: EventType;
  Error: UInt16;
begin
  RctSetRectangle(WorkRect, 0, 16, 160, 128);
  FrmGotoForm(MainForm);
  Repeat
    EvtGetEvent(Event, 5);
    if not SysHandleEvent(Event) then
      if not MenuHandleEvent(MyMenu,Event,Error) then
        if not FrmDispatchEvent(Event) then
          if not HandleEvent(Event) then 
            if (FormLoad and FormOpen) then 
              if TurnsToPlay>0 then begin
                DoCalc;
                dec(TurnsToPlay); 
              end
              else 
                DoStop:=True;
  Until DoStop; // or (Event.eType=appStopEvent);
//  SavePreferences;
  if FrmGetActiveForm<>NIL then begin
    FrmEraseForm(FrmGetActiveForm);
    FrmDeleteForm(FrmGetActiveForm);
  end;
end;

Procedure SpeedBeg;
begin
  RndSeed:=0.1234;
  TurnsToPlay:=1900;
  CReset;
  NumMem:=0;
end;

Procedure SpeedEnd;
Var
  N: Integer;
begin
  N:=StopTimer;
  N:=frmCustomAlert(ResultWindow,'SNumerus test lasted ',i2s(N),' seconds.');
end;

begin
  StartTimer;
//  LoadPreferences;
  InitCalc;
  SpeedBeg;
  Main;
  SpeedEnd;
end.
