/*
 * teezee.c
 * Copyright (C) 1998 William H. Ball
 * Copyright (C) 2003 Daniel Thompson <tzones@redfelineninja.org.uk>
 *
 * This file is part of TZones - A simple world clock for PalmOS
 *
 * TZones is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * TZones is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include <PalmCompatibility.h>

#include "clock.h"
#include "graphics.h"
#include "teezee.h"

#define elementsof(x) (sizeof(x) / sizeof((x)[0]))

#define appFileCreator    '8843'
#define appVersionNumber  20100

typedef struct TZCity {
	char *name;
	short gmtOffset;
	Boolean  hasDaylightSaving;
} TZCityType;

static TZCityType tzCitys[] = {
#define h *60+
	{ "Africa/Cairo",		  2 h 00, true  },
	{ "Africa/Casablanca",		  0 h 00, false },
	{ "Africa/Cape Town",		  2 h 00, false },
	{ "Africa/Johannesburg",	  2 h 00, false },
	{ "Africa/Timbuktu",		  0 h 00, false },
	{ "Africa/Tunis",		  1 h 00, false },
	{ "America/Anchorage",		 -9 h 00, true  },
	{ "America/Boulder",		 -7 h 00, true  },
	{ "America/Boston",		 -5 h 00, true  },
	{ "America/Chicago",		 -6 h 00, true  },
	{ "America/Dallas",		 -6 h 00, true  },
	{ "America/Honolulu",		-10 h 00, false },
	{ "America/Houston",		 -6 h 00, true  },
	{ "America/Jamaica",		 -5 h 00, false },
	{ "America/Las Vegas",		 -8 h 00, true  },
	{ "America/Miami",		 -5 h 00, true  },
	{ "America/New York",		 -5 h 00, true  },
	{ "America/Phila",		 -5 h 00, true  },
	{ "America/Rio",		 -3 h 00, false },
	{ "America/Seattle",		 -8 h 00, true  },
	{ "America/San Fran",		 -8 h 00, true  },
	{ "America/Wash., DC",		 -5 h 00, true  },
	{ "Asia/Baghdad",		  3 h 00, true  },
	{ "Asia/Bangkok",		  7 h 00, false },
	{ "Asia/Beijing",		  8 h 00, false },
	{ "Asia/Calcutta",		  5 h 30, false },
	{ "Asia/Damascus",		  3 h 00, true  },
	{ "Asia/Hong Kong",		  8 h 00, false },
	{ "Asia/Irkutsk",		  9 h 00, true  },
	{ "Asia/Jerusalem",		  3 h 00, true  },
	{ "Asia/Kabul",			  4 h 30, false },
	{ "Asia/Kuwait",		  3 h 00, false },
	{ "Asia/Manila",		  8 h 00, false },
	{ "Asia/Osaka",			  9 h 00, false },
	{ "Asia/Seoul",			  9 h 00, false },
	{ "Asia/Singapore",		  8 h 00, false },
	{ "Asia/Tokyo",			  9 h 00, false },
	{ "Australia/Melbourne",	 10 h 00, false },
	{ "Australia/Canberra",		 10 h 00, false },
	{ "Australia/Perth",		  8 h 00, false },
	{ "Europe/Amsterdam",		  1 h 00, true  },
	{ "Europe/Berlin",		  1 h 00, true  },
	{ "Europe/Brussels",		  1 h 00, true  },
	{ "Europe/Dublin",		  0 h 00, true  },
	{ "Europe/Frankfurt",		  1 h 00, true  },
	{ "Europe/Helsinki",		  2 h 00, true  },
	{ "Europe/Istanbul",		  2 h 00, true  },
	{ "Europe/London",		  0 h 00, true  },
	{ "Europe/Madrid",		  1 h 00, true  },
	{ "Europe/Milan",		  1 h 00, true  },
	{ "Europe/Moscow",		  3 h 00, true  },
	{ "Europe/Paris",		  1 h 00, true  },
	{ "Europe/Rome",		  1 h 00, true  },
	{ "Europe/Stockholm",		  1 h 00, true  },
	{ "Pacific/Auckland",		 12 h 00, false },
	{ "Pacific/Fiji",		 12 h 00, false },
	{ "Pacific/Honolulu",		-10 h 00, false }
#undef h
};

static short status = ON;	/* animate? */
static short hour = 0;
static short minute = 0;
static short oldminute = 0;	/* one-minute update flag */

static short LOFFSET;		/* local GMTOFFSET */
static short DST;		/* default Daylight Saving Time OFF */
static short SHOWUTC;		/* default display of GMT,UTC OFF */
static short reverse;		/* draw clocks in reverse */
static short tzCityId[4];

#define TZ_CITY_NAME(idx) (tzCitys[tzCityId[idx]].name)
#define TZ_CITY_SHORT_NAME(idx) \
	(StrChr(tzCitys[tzCityId[idx]].name, '/') ? \
	 StrChr(tzCitys[tzCityId[idx]].name, '/') + 1 : \
	 tzCitys[tzCityId[idx]].name)
#define TZ_CITY_GMT_OFFSET(idx) (tzCitys[tzCityId[idx]].gmtOffset)
#define TZ_CITY_HAS_DAYLIGHT_SAVING(idx) (tzCitys[tzCityId[idx]].hasDaylightSaving)

static short counter = 29;	/* animation counter */

/* storage for our preferences */
typedef struct {
	int LOFFSETpref;	/* our local GMT offset */
	int DSTpref;		/* use Daylight Savings Time? */
	int SHOWUTCpref;	/* display UCT,GMT,UTC? */
	int REVERSEpref;	/* black or white clock */
	short cityId[4];
} TZonepreftype;

/* draw time and date on screen */
static void displaydatetime()
{
	DateTimeType ct;
	short width, x;		/* used to center the strings */
	static char ztime[32], zdate[32];	/* placeholders for time,date */
	static char ldate[32];

	SystemPreferencesType sysPrefs;

	/* get prefs */
	PrefGetPreferences(&sysPrefs);

	x = 0;
	width = 0;
	hour = 0;
	minute = 0;

	ztime[0] = 0;
	zdate[0] = 0;
	ldate[0] = 0;

	FntSetFont(largeFont);	/* use large font as default */

	/* fill out date,time structure */
	TimSecondsToDateTime(TimGetSeconds(), &ct);

	hour = ct.hour;
	minute = ct.minute;

	/* fill in string with date, center, then draw */
	DateToAscii(ct.month, ct.day, ct.year, sysPrefs.dateFormat, zdate);
	width = FntCharsWidth(zdate, StrLen(zdate));
	x = ((160 - width) / 2);
	if (status != OFF)
		WinDrawChars(zdate, StrLen(zdate), x, 118);

	/* fill in string with time, center, then draw */
	TimeToAscii(ct.hour, ct.minute, sysPrefs.timeFormat, ztime);
	width = FntCharsWidth(ztime, StrLen(ztime));
	x = ((160 - width) / 2);
	if (status != OFF)
		WinDrawChars(ztime, StrLen(ztime), x, 90);

	/* get long format date */
	DateToDOWDMFormat(ct.month, ct.day, ct.year, sysPrefs.dateFormat,
			  ldate);
	if (StrStr(ldate, "Sun") != NULL)
		StrCopy(ldate, "Sunday");
	if (StrStr(ldate, "Mon") != NULL)
		StrCopy(ldate, "Monday");
	if (StrStr(ldate, "Tue") != NULL)
		StrCopy(ldate, "Tuesday");
	if (StrStr(ldate, "Wed") != NULL)
		StrCopy(ldate, "Wednesday");
	if (StrStr(ldate, "Thu") != NULL)
		StrCopy(ldate, "Thursday");
	if (StrStr(ldate, "Fri") != NULL)
		StrCopy(ldate, "Friday");
	if (StrStr(ldate, "Sat") != NULL)
		StrCopy(ldate, "Saturday");

	FntSetFont(stdFont);
	width = FntCharsWidth(ldate, StrLen(ldate));
	x = ((160 - width) / 2);
	if (status != OFF)
		WinDrawChars(ldate, StrLen(ldate), x, 105);
}

/* draw clocks 
 * draw four clocks on screen with name of city, clockface w/hands, and time 
 */
static void drawclocks()
{
	short dhour, dminute, i, thour, tminute, x, width;

	thour = hour - (DST == ON ? 1 : 0) - (LOFFSET / 60);
	tminute = minute - (LOFFSET % 60);
	tminute = (tminute < 0   ? thour--, tminute + 60 :
	           tminute >= 60 ? thour++, tminute - 60 : tminute);
	thour = (thour < 0   ? thour + 24 : thour >= 24 ? thour - 24 : thour);

	
	
	/* all drawing in this function uses the standard font */
	FntSetFont(stdFont);
	
	/* are we using Daylight Savings Times? */
	if (DST == ON) {
		const static char *dstmsg1 = "dst";
		
		width = FntCharsWidth(dstmsg1, StrLen(dstmsg1));
		x = ((160 - width) / 2);
		WinDrawChars(dstmsg1, StrLen(dstmsg1), x, 145);	/* yup, display reminder */
	}

	/* show UTC time? */
	if (SHOWUTC == ON) {
		char utchour[6];
		
		StrPrintF(utchour, "%02d:%02d", thour, tminute);
		width = FntCharsWidth(utchour, StrLen(utchour));
		WinDrawChars(utchour, StrLen(utchour), ((160 - width) / 2), 130);
	}

	/* now draw the clocks */
	for (i=0; i<4; i++) {
		const static PointType names[4]  = {{  3,  20}, {  8,  85}, {120,  20}, {115,  85}};
		const static PointType clocks[4] = {{ 20,  50}, { 25, 115}, {140,  50}, {134, 115}};
		
		/* calculate time by adding clock 1's GMTOFFSET */
		dhour = thour + TZ_CITY_GMT_OFFSET(i) / 60;
		dhour += (DST == ON && TZ_CITY_HAS_DAYLIGHT_SAVING(i) ? 1 : 0);
		dminute = tminute + TZ_CITY_GMT_OFFSET(i) % 60;
		dminute = (dminute < 0   ? dhour--, dminute + 60 :
		           dminute >= 60 ? dhour++, dminute - 60 : dminute);
		dhour = (dhour < 0   ? dhour + 24 : dhour >= 24 ? dhour - 24 : dhour);
	
		WinDrawChars(TZ_CITY_SHORT_NAME(i), StrLen(TZ_CITY_SHORT_NAME(i)), names[i].x, names[i].y);
		drawclock(clocks[i].x, clocks[i].y, 13, dhour, dminute, reverse);		
	}
}

/* erase local date, time area 
 * used between updates, clock face changes, etc.
 */
static void erasedatetime()
{
	RectangleType r;

	/* erase rect r */
	r.topLeft.x = 30;
	r.topLeft.y = 100;
	r.extent.x = 135;
	r.extent.y = 130;

	WinEraseRectangle(&r, 0);
}

/* the sole purpose of this event handler is to permit the use of the
 * scroll keys in the Cities dialog
 */
static Boolean SelectDialogEventHandler(EventPtr event)
{
	Boolean handled = true;
	WinDirectionType direction;
	FormPtr form;
	Int16 listID;
	void *listPtr;
	Int16 visibleItems;
	
	switch (event->eType) {
	case keyDownEvent:
		direction = winDown;
		switch (event->data.keyDown.chr) {
		case vchrPageUp:
			direction = winUp;
			/*FALLTHRU*/
		case vchrPageDown:
			form = FrmGetActiveForm();
			listID = FrmGetObjectIndex(form, listID_select);
			listPtr = (ListPtr) FrmGetObjectPtr(form, listID);
			visibleItems = LstGetVisibleItems (listPtr);
			if (!LstScrollList (listPtr, direction, visibleItems-1)) {
				SndPlaySystemSound (sndWarning);
			}
			break;
		default:
			handled = false;
		}
		break;
	default:
		handled = false;
		break;
	}
	
	return handled;
}

static Int16 DoSelectDialog(Int16 initialSelection)
{
	FormPtr form;
	Int16 buttonID, listID;
	ListPtr listPtr;
	int i;
	Char *listChoices[elementsof(tzCitys)];
	Int16 result;
	
	form = FrmInitForm(formID_select);
	FrmSetEventHandler(form, SelectDialogEventHandler);
	listID = FrmGetObjectIndex(form, listID_select);
	listPtr = (ListPtr) FrmGetObjectPtr(form, listID);
	for (i=0; i<elementsof(tzCitys); i++) {
		listChoices[i] = tzCitys[i].name;
	}
	LstSetListChoices(listPtr, listChoices, elementsof(tzCitys));
	LstSetSelection(listPtr, initialSelection);
	LstMakeItemVisible(listPtr, initialSelection);
	buttonID = FrmDoDialog(form);
	if (buttonID != buttonID_cancel) {
		result = LstGetSelection(listPtr);
	} else {
		result = -1;
	}
	FrmDeleteForm(form);
	return result;
}

static void ClocksDialogUpdateLabels(FormPtr form)
{
	int i;
	
	for (i=0; i<4; i++) {
		Int16 clockID = FrmGetObjectIndex(form,
			0 == i ? selectorID_topLeft :
			1 == i ? selectorID_bottomLeft :
			2 == i ? selectorID_topRight :
			         selectorID_bottomRight);
		void *controlPtr = FrmGetObjectPtr (form, clockID);
		
		CtlSetLabel (controlPtr, TZ_CITY_SHORT_NAME(i));
	}
}

static Boolean ClocksDialogEventHandler(EventPtr event)
{
	Boolean handled = true;
	Int16 id = -1;
	Int16 res;
	
	switch (event->eType) {
	case ctlSelectEvent:
		switch (event->data.ctlSelect.controlID) {
		case selectorID_bottomRight:	id++;	/*FALLTHRU*/
		case selectorID_topRight:	id++;	/*FALLTHRU*/
		case selectorID_bottomLeft:	id++;	/*FALLTHRU*/
		case selectorID_topLeft:	id++;
			res = DoSelectDialog(tzCityId[id]);
			if (-1 != res) {
				tzCityId[id] = res;
				ClocksDialogUpdateLabels(FrmGetActiveForm());
			}
			break;
		default:
			handled = false;
			break;
		}
		break;
	default:
		handled = false;
		break;
	}
	
	return handled;
}

static void DoClocksDialog(void)
{
	FormPtr form;
	Int16 buttonID;
	short cityIdBackup[4];

	/* backup the original clock settings so these can be restored if
	 * the user cancels the clocks dialog
	 */
	MemMove (cityIdBackup, tzCityId, sizeof(cityIdBackup));
	
	form = FrmInitForm(formID_clocks);
	FrmSetEventHandler(form, ClocksDialogEventHandler);
	ClocksDialogUpdateLabels(form);
	buttonID = FrmDoDialog(form);
	
	if (buttonID == buttonID_cancel) {
		MemMove (tzCityId, cityIdBackup, sizeof(cityIdBackup));
	}
	
	FrmDeleteForm(form);
}

static Boolean PreferencesDialogEventHandler(EventPtr event)
{
	Boolean handled = true;
	Int16   cityID;
	FormPtr form;
	Int16   controlID;
	void *controlPtr;
	
	switch (event->eType) {
	case ctlSelectEvent:
		switch (event->data.ctlSelect.controlID) {
		case selectorID_localTime:
			cityID = DoSelectDialog(0);
			if (-1 != cityID) {
				form = FrmGetActiveForm();
				controlID = FrmGetObjectIndex(form, selectorID_localTime);
				controlPtr = FrmGetObjectPtr (form, controlID);
				CtlSetLabel (controlPtr, tzCitys[cityID].name);
				LOFFSET = tzCitys[cityID].gmtOffset;
			}
			break;
		default:
			handled = false;
			break;
		}
		break;
	default:
		handled = false;
		break;
	}
	
	return handled;
}

static void DoPreferencesDialog(void)
{
	FormPtr form;
	Int16 utcID, dstID, reverseID, buttonID;
	Int16 oldGmtOffset;
	
	/* initialize the form and get the IDs of the checkboxes within it */		
	form = FrmInitForm(formID_preferences);
	oldGmtOffset = LOFFSET;
	FrmSetEventHandler(form, PreferencesDialogEventHandler);
	utcID = FrmGetObjectIndex(form, checkboxID_displayUTC);
	dstID = FrmGetObjectIndex(form, checkboxID_daylightSaving);
	reverseID = FrmGetObjectIndex(form, checkboxID_alternativeClockFaces);

	/* set the checkboxes to match the existing constants */
	FrmSetControlValue(form, utcID, SHOWUTC);
	FrmSetControlValue(form, dstID, DST);
	FrmSetControlValue(form, reverseID, !reverse); 
			
	buttonID = FrmDoDialog(form);
	
	/* if the form was dismissed with the OK button then harvest the new
	 * settings
	 */
	if (buttonID != buttonID_cancel) {	
		SHOWUTC = FrmGetControlValue(form, utcID);
		DST = FrmGetControlValue(form, dstID);
		reverse = !FrmGetControlValue(form, reverseID);
	} else {
		LOFFSET = oldGmtOffset;
	}
	
	FrmDeleteForm(form);
}

/* do an easter egg - print a random saying at the bottom of the screen
 * when the app detects a pendown event at bottom of screen
 */
static void DoEasterEgg(void)
{
	static const char *messages[] = {
		"Random core dump. Please reboot now.",
		"Don't you just love your Palm?",
		"Make love, not war!",
		"Yabba dabba do!",
		"A mushroom cloud has no silver lining.",
		"Do be do be do, baby!",
		"Absolutum obsoletum",
		"Let sleeping dogs lie.",
		"Yes!",
		"No!",
		"Maybe.",
		"Try again.",
		"You'll have a great day.",
		"A mind is a wonderful thing to waste.",
		"Beware of geeks bearing grifts.",
		"Loop before you leak.",
		"Look before you leap.",
		"No good deed goes unpunished.",
		"YOW! Am I in Cleveland? - Zippy",
		"You will find happiness soon.",
		"You will find happiness someday.",
		"Having great fun - wish you were here.",
		"Empty what is full. Fill what is empty.",
		"To err is human, to 'Moo!' bovine.",
		"Proceed directly to jail. Do not pass go.",
		"Trust in Allah, but tie your camel.",
		"Play safely. Accidents cause people.",
		"f u cn rd ths u cn b a cmptr pgrmr.",
		"You go first.",
		"We're all bozos on this bus.",
		"Love is a grave mental disease.",
		"It's shagadelic, baby!",
		"A waist is a terrible thing to mind.",
		"Boycott meat -- suck your thumb.",
		"Time for lunch!",
		"Are we there yet?",
		"GNU/Linux - it's not just for breakfast!",
		"GNU rules, OK?",
		"This app brought to you by GNU/Linux!",
		"Use GNU/Linux - feel better!",
		"GNU/Linux rules, OK?",
		"GNU/Linux. The quicker picker upper.",
		"Guerrila computing - Linux!",
		"Join the FSF - http://member.fsf.org",
		"Why is abbreviation such a long word?",
		"PalmOS - Bonsai computing!"
	};

	static const RectangleType easterEggArea = { { 0, 145 }, { 160, 15 } };
	const char *msg;
	short width, x;
	
	/* randomly select a message */
	msg = messages[SysRandom(0) % (sizeof(messages) / sizeof(messages[0]))];
	
	/* draw the message centered within the easter egg area */
	width = FntCharsWidth(msg, StrLen(msg));
	x = ((easterEggArea.extent.x - width) / 2);
	WinEraseRectangle (&easterEggArea, 0);
	WinDrawChars(msg, StrLen(msg), easterEggArea.topLeft.x + x, easterEggArea.topLeft.y);
}

/* Handle a pen click */
static Boolean PenEvent(EventPtr event)
{
	ErrNonFatalDisplayIf(event->eType != penDownEvent, "PenEvent called illegally");
	
	status = ON;	/* draw when pen is down */
	
	/* has user tapped bottom of screen area? */
	if (140 <= event->screenY && event->screenY < 160) {
		DoEasterEgg();	/* do random saying */
		return true;
	}
	
	return false;
}

/* handle our events */
static Boolean MainFormEventHandler(EventPtr event)
{
	FormPtr form = FrmGetActiveForm();
	int handled = 1;
	int redrawClocks;

	switch (event->eType) {
	case frmOpenEvent:
		FrmDrawForm(form);

		/* load bitmap resources for our app */
		setEarth();
		break;
	case menuEvent:
		redrawClocks = 1;

		switch (event->data.menu.itemID) {
		case menuitemID_preferences:
			DoPreferencesDialog();
			break;
		case menuitemID_clocks:
			DoClocksDialog();
			break;
		case menuitemID_about:
			form = FrmInitForm(formID_about);
			FrmDoDialog(form);
			FrmDeleteForm(form);
			redrawClocks = 0;
			break;
		
		}

		if (redrawClocks) {
			eraseclocks();
			drawclocks();
		}

		status = ON;
		break;

	case nilEvent:
		displaydatetime();
		if (status == OFF)
			break;	/* don't draw */
		/* display one frame */

		if (counter == 0) {
			spinEarth(counter);
			counter = 29;
		} else {
			spinEarth(counter);
			--counter;
		}

		/* only update clocks once a minute */
		if (oldminute == minute)
			break;
		eraseclocks();
		erasedatetime();
		oldminute = minute;
		drawclocks();
		/* showEarth(); *//* mercator projection */
		break;

	case penDownEvent:
		handled = PenEvent(event);
		break;

	case winExitEvent:
		/* stop animating the globe when we focus leaves the main form (this is typically
		 * because we have switched to viewing a menu
		 */
		if (event->data.winExit.exitWindow == (WinHandle) form) {
			status = OFF;
		}
		break;

	case winEnterEvent:
		/* start animating the globe when focus returns to the main form */
		if (event->data.winEnter.enterWindow == (WinHandle) form &&
		    event->data.winEnter.enterWindow ==
		    (WinHandle) FrmGetFirstForm()) {
			status = ON;
		}
		break;

	default:
		handled = 0;
		break;
	}
	return handled;
}

/* handle menu calls, etc. */
static void MainLoop(void)
{
	short err;
	int formID;
	FormPtr form;
	EventType event;

	while (event.eType != appStopEvent) {
		EvtGetEvent(&event, 10);

		if (SysHandleEvent(&event)) {
			continue;
		} else if (MenuHandleEvent((void *) 0, &event, &err)) {
			continue;
		} else if (event.eType == frmLoadEvent) {
			formID = event.data.frmLoad.formID;
			form = FrmInitForm(formID);
			FrmSetActiveForm(form);
			switch (formID) {
			case formID_main:
				FrmSetEventHandler(form, MainFormEventHandler);
				break;
			}
		} else {
			FrmDispatchEvent(&event);
		}
	}
}

/* load prefs */
static void StartApplication(void)
{
	TZonepreftype *savedPrefsP;
	Int16 savedPrefsSize;

	savedPrefsP = MemPtrNew(sizeof(TZonepreftype));
	savedPrefsSize = sizeof(TZonepreftype);
	if ((noPreferenceFound != PrefGetAppPreferences(appFileCreator, appVersionNumber,
				  			savedPrefsP, &savedPrefsSize, true)) &&
	    (savedPrefsSize == sizeof(TZonepreftype)))
	{
		LOFFSET = savedPrefsP->LOFFSETpref;
		DST = savedPrefsP->DSTpref;
		SHOWUTC = savedPrefsP->SHOWUTCpref;
		reverse = savedPrefsP->REVERSEpref;
		tzCityId[0] = savedPrefsP->cityId[0];
		tzCityId[1] = savedPrefsP->cityId[1];
		tzCityId[2] = savedPrefsP->cityId[2];
		tzCityId[3] = savedPrefsP->cityId[3];

	} else {
		EventType e;
		
		/* if the size of the preferences is wrong then delete them and restore the defaults */
		if (savedPrefsSize != sizeof(TZonepreftype)) {
			PrefSetAppPreferences(appFileCreator,
					      appVersionNumber,
					      appVersionNumber, NULL, 0,
					      true);
		}

		/* default values when app is first run */
		DST = OFF;		/* don't use DST */
		SHOWUTC = OFF;		/* don't display UTC,GMT */
		LOFFSET = -5 * 50;	/* our local offset */
		reverse = ON;		/* use reverse clocks */
		tzCityId[0] = 21;	/* clock1 London */
		tzCityId[1] = 27;	/* clock2 Moscow */
		tzCityId[2] = 28;	/* clock3 New York */
		tzCityId[3] = 30;	/* clock4 Paris */

		/* we really want the user to be presented with the preferences
		 * dialog so they can pick their local time zone but unfortunately
		 * we cannot yet call DoPreferencesDialog() (since we need
		 * to process the frmLoadEvent for the main form before it
		 * is safe to do that). we can however queue up a fake menu
		 * event. note that we have cheated and not filled in all the
		 * values because we know exactly how MainFormEventHandler will
		 * process it.
		 */
		e.eType = menuEvent;
		e.data.menu.itemID = menuitemID_preferences;
		EvtAddEventToQueue (&e);
	}
	MemPtrFree(savedPrefsP);
}

/* save the currently selected preferences */
static void StopApplication(void)
{
	TZonepreftype *savedPrefsP;

	savedPrefsP = MemPtrNew(sizeof(TZonepreftype));
	savedPrefsP->LOFFSETpref = LOFFSET;
	savedPrefsP->DSTpref = DST;
	savedPrefsP->SHOWUTCpref = SHOWUTC;
	savedPrefsP->REVERSEpref = reverse;
	savedPrefsP->cityId[0] = tzCityId[0];
	savedPrefsP->cityId[1] = tzCityId[1];
	savedPrefsP->cityId[2] = tzCityId[2];
	savedPrefsP->cityId[3] = tzCityId[3];

	PrefSetAppPreferences(appFileCreator, appVersionNumber,
			      appVersionNumber, savedPrefsP,
			      sizeof(TZonepreftype), true);
	MemPtrFree(savedPrefsP);

	/* OK to clean up here? */
	FrmDeleteForm(FrmGetActiveForm());
	FrmCloseAllForms();

}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	if (cmd == sysAppLaunchCmdNormalLaunch) {
		FrmGotoForm(formID_main);
		StartApplication();	/* load prefs */
		MainLoop();
		StopApplication();	/* save prefs */
	}
	return 0;
}
